@@ -5,16 +5,26 @@ def handle_error(resp):
     if resp.status_code == 400:
         if resp.reason == 'AuthorizationError':
             raise AuthorizationError(resp.text)
-        elif resp.reason == 'InputError':
+        if resp.reason == 'InputError':
             raise InputError(resp.text)
-        elif resp.reason == 'DuplicateRecordError':
+        if resp.reason == 'DuplicateRecordError':
             raise DuplicateRecordError(resp.text)
     if resp.status_code == 403:
         raise ForbiddenError(resp.text)
-    elif resp.status_code == 404:
+    if resp.status_code == 404:
         raise NotFoundError(resp.text)
-    else:
-        raise RuntimeError(resp.text)
+    raise RuntimeError(resp.text)
+
+
+def handle_text_resp(func):
+    def output_trans(resp):
+        if resp.status_code == 200:
+            return resp.text
+        else:
+            handle_error(resp)
+
+    output_trans.content_type = 'text'
+    return output_trans
 
 
 def handle_json_resp(func):
@@ -26,3 +36,14 @@ def handle_json_resp(func):
 
     output_trans.content_type = 'json'
     return output_trans
+
+
+def handle_content_resp(func):
+    def output_trans(resp):
+        if resp.status_code == 200:
+            return resp.content
+        else:
+            handle_error(resp)
+
+    output_trans.content_type = 'content'
+    return output_trans
